import pyGenericIO
import SepVector
import Hypercube
storageConvert={"dataFloat": pyGenericIO.dataType.dataFloat,
"dataInt": pyGenericIO.dataType.dataInt,
"dataByte": pyGenericIO.dataType.dataByte,
"dataComplex": pyGenericIO.dataType.dataComplex,
"dataShort": pyGenericIO.dataType.dataShort,

"dataDouble": pyGenericIO.dataType.dataDouble
}
usageConvert= {"usageIn":  pyGenericIO.usage_code.usageIn,
      "usageOut": pyGenericIO.usage_code.usageOut,
    "usageInOut":  pyGenericIO.usage_code.usageInOut,
      "usageScr":  pyGenericIO.usage_code.usageScr};

ioModes=pyGenericIO.ioModes([""])


class pythonParams:
	def __init__(self,inPars):
		"""Create a parameter object that reads from a python dictionary"""
		self.pars={}
		self.inPars=inPars
		for k,v in self.inPars.items():
			found=False;
			if isinstance(v, int) or isinstance(v, float)  or isinstance(v, str):
				vout=str(v)
				found=True
			elif isinstance(v,list):
				d=None
				found=True
				for l in v:
					if not d:
						d=l
					else:
						d+=","+l
			if found:
				self.pars[k]=vout
		self.cppMode=pyGenericIO.pythonParams(self.pars)
	def getCpp(self):
		return self.cppMode


class regFile:
	def __init__(self,ioM,tag, **kw):
		"""Get regular file python object
			Requiered:
			    iom  - IO mode
				tag  - Tag for file
			Optional:
				fromHyper - Hypercube describing file (also must specify storage)
				storage  - float,complex,byte,double, or int (defaults to float)
				fromVector - Vector 
				usage - Defaults to IN for from file OUT for everything else
		"""
		self.tag=tag
		self.usage=None
		if "usage" in kw:
			if kw["usage"]=="INPUT": 
				self.usage="UsageIn"
			elif kw["usage"]=="output":
				self.usage="usageOut"
			elif kw["usage"]=="inout":
				self.usage="usageInOut"
			else:
				raise Exception("Only understand input,output, and inout for usage")
		self.storage="dataFloat"
		if "storage" in kw:
			if kw["storage"]=="float":
				self.storage="dataFloat"
			elif kw["storage"]=="int":
				self.storage="dataInt"
			elif kw["storage"]=="byte":
				self.storage="dataByte"
			elif kw["storage"]=="double":
				self.storage="dataDouble"
			elif kw["storage"]=="complex":
				self.storage="dataComplex"
			else:
				raise Exception("Only understand float,int,double,byte,and complex for storage")


		if "fromHyper" in kw:
			if not isinstance(kw["fromHyper"],Hypercube.hypercube):
				raise Exception("Must pass a hypercube object when creating a file from a hypercube")
			if not self.usage:
				self.usage="usageOut"
			elif self.usage=="usageIn":
				raise Exception("Can not have usageIn when creating from Hypercube")
			self.cppMode=ioM.getRegFile(self.tag,usageConvert[self.usage])
			self.cppMode.setHyper(kw["fromHyper"].getCpp())
			self.cppMode.setDataType(storageConvert[self.storage])
			self.cppMode.writeDescrption()
		elif "fromVector" in kw:
			if not isinstance(kw["fromVector"],SepVector.vector):
				raise Exception("When creating a file from a vector must be inherited from SepVector.vector")
			self.storage=kw["fromVector"].getStorageType()
			if not self.usage:
				self.usage="usageOut"
			elif self.usage=="usageIn":
				raise Exception("Can not have usageIn when creating from Hypercube")
			self.cppMode=ioM.getRegFile(self.tag,usageConvert[self.usage])
			self.cppMode.setHyper(kw["fromVector"].getCpp().getHyper())
			self.cppMode.setDataType(storageConvert[self.storage])
			self.cppMode.writeDescription()
		else: #Assuming from file
			if not self.usage:
				self.usage="usageIn"
			elif self.usage=="usageOut":
				raise Exception("Can not specify usageOut when creating from a file")
			self.cppMode=ioM.getRegFile(self.tag,usageConvert[self.usage])
			self.cppMode.readDescription()
	def getInt(self,tag,*arg):
		"""Get integer from a given IO"""
		if(len(arg)==1):
			return self.cppMode.getInt(tag,arg)
		return self.cppMode.getInt(tag)
	def getFloat(self,tag,*arg):
		"""Get a float from a given IO"""
		if(len(arg)==1):
			return self.cppMode.getFloat(tag,arg)
		return self.cppMode.getFloat(tag)
	def getBool(self,tag,*arg):
		"""Get a boolean from a given IO"""
		if(len(arg)==1):
			return self.cppMode.getBool(tag,arg)
		return self.cppMode.getBool(tag)
	def getString(self,tag,*arg):
		"""Get a string from a given IO"""
		if(len(arg)==1):
			return self.cppMode.getString(tag,arg)
		return self.cppMode.getString(tag)
	def getInts(self,tag,*arg):
		"""Get integers from a given IO"""
		if(len(arg)==1):
			return self.cppMode.getInts(tag,arg)
		return self.cppMode.getInts(tag)
	def getInts(self,tag,*arg):
		"""Get integers from a given IO"""
		if(len(arg)==1):
			return self.cppMode.getInts(tag,arg)
		return self.cppMode.getInts(tag)
	def getFloats(self,tag,*arg):
		"""Get floats from a given IO"""
		if(len(arg)==1):
			return self.cppMode.getFloats(tag,arg)
		return self.cppMode.getFloats(tag)
	def close(self):
		"""Close file"""
		self.cppMode.close()
	def putInt(self,tag,arg):
		"""Put a float into description"""
		self.cppMode.putInt(tag,arg)
	def putFloat(self,tag,arg):
		"""Put a float into description"""
		self.cppMode.putFloat(tag,arg)
	def putString(self,tag,arg):
		"""Put a string into description"""
		self.cppMode.putString(tag,arg)
	def putBool(self,tag,arg):
		"""Put a boolean into description"""
		self.cppMode.putBool(tag,arg)
	def putFloats(self,tag,arg):
		"""Put floats into description"""
		self.cppMode.putFloats(tag,arg)
	def putInts(self,tag,arg):
		"""Put ints into description"""
		self.cppMode.putInts(tag,arg)
	def getCpp(self):
		"""Return cpp object"""
		return self.cppMode
	def getHyper(self):
		"""Return hypercube for the file"""
		return Hypercube.hypercube(hypercube=self.cppMode.getHyper())
class io:
	def __init__(self,*arg):
		"""Initialize IO"""
		if len(arg)>0:
			self.cppMode=ioModes.getIO(arg)
		else:
			self.cppMode=ioModes.getDefaultIO()
		self.param=self.cppMode.getParamObj()
	def getInt(self,tag,*arg):
		"""Get integer from a given IO"""
		if(len(arg)==1):
			return self.param.getInt(tag,arg)
		return self.param.getInt(tag)
	def getFloat(self,tag,*arg):
		"""Get a float from a given IO"""
		if(len(arg)==1):
			return self.param.getFloat(tag,arg)
		return self.param.getFloat(tag)
	def getBool(self,tag,*arg):
		"""Get a boolean from a given IO"""
		if(len(arg)==1):
			return self.param.getBool(tag,arg)
		return self.param.getBool(tag)
	def getString(self,tag,*arg):
		"""Get a string from a given IO"""
		if(len(arg)==1):
			return self.param.getString(tag,arg)
		return self.param.getString(tag)
	def getInts(self,tag,*arg):
		"""Get integers from a given IO"""
		if(len(arg)==1):
			return self.param.getInts(tag,arg)
		return self.param.getInts(tag)
	def getInts(self,tag,*arg):
		"""Get integers from a given IO"""
		if(len(arg)==1):
			return self.param.getInts(tag,arg)
		return self.param.getInts(tag)
	def getFloats(self,tag,*arg):
		"""Get floats from a given IO"""
		if(len(arg)==1):
			return self.param.getFloats(tag,arg)
		return self.param.getFloats(tag)
	def getRegFile(self,tag,**kw):
		"""Get regular file python object
			Requiered:
				tag  - Tag for file
			Optional:
				fromHyper - Hypercube describing file (also must specify storage)
				storage  - float,complex,byte,double, or int (defaults to float)
				fromVector - Vector 
		"""
		return regFile(self.cppMode,tag,**kw)
	def getVector(self,tag):
		"""Get vector from a file and read its contents"""
		file=self.getRegFile(tag)
		hyper=file.getHyper()
		vec=SepVector.getSepVector(hyper)
		if file.storage=="dataFloat":
			file.getCpp().readFloatStream(vec.getCpp())
		elif file.storage=="dataComplex":
			file.getCpp().readComplexStream(vec.getCpp())
		elif file.storage=="dataByte":
			file.getCpp().readByteStream(vec.getCpp())
		elif file.storage=="dataInt":
			file.getCpp().readIntStream(vec.getCpp())		
		elif file.storage=="dataDouble":
			file.getCpp().readDoubleStream(vec.getCpp())
		file.close()
		return vec
	def writeVector(self,tag,vec):
		"""Write entire sepVector to disk
		   tag - File to write to
		   vec - Vector to write"""
		file=regFile(self.cppMode,tag,fromVector=vec)
		if file.storage=="dataFloat":
			file.getCpp().writeFloatStream(vec.getCpp())
		elif file.storage=="dataComplex":
			file.getCpp().writeComplexStream(vec.getCpp())
		elif file.storage=="dataByte":
			file.getCpp().writeByteStream(vec.getCpp())
		elif file.storage=="dataInt":
			file.getCpp().writeIntStream(vec.getCpp())		
		elif file.storage=="dataDouble":
			file.getCpp().writeDoubleStream(vec.getCpp())
		return vec	

defaultIO=io()
